@@ -1186,4 +1186,5 @@ def _compute_n_welch_segments(n_times, method_kw):
     # sanity check values / replace `None`s with real numbers
     n_fft, n_per_seg, n_overlap = _check_nfft(n_times, **_defaults)
     # compute expected number of segments
-    return n_times // (n_per_seg - n_overlap)
+    step = n_per_seg - n_overlap
+    return (n_times - n_overlap) // step
