@@ -8,6 +8,7 @@ from copy import deepcopy
 
 import matplotlib
 import matplotlib.pyplot as plt
+from matplotlib import backend_bases
 import numpy as np
 import pytest
 from numpy.testing import assert_allclose
@@ -16,9 +17,11 @@ from mne import Annotations, create_info, pick_types
 from mne.annotations import _sync_onset
 from mne.datasets import testing
 from mne.io import RawArray
-from mne.utils import _dt_to_stamp, _record_warnings, get_config, set_config
+from mne.io.pick import _DATA_CH_TYPES_ORDER_DEFAULT, _PICK_TYPES_DATA_DICT
+from mne.utils import (_dt_to_stamp, _record_warnings, get_config, set_config,
+                       _assert_no_instances)
 from mne.viz import plot_raw, plot_sensors
-from mne.viz.utils import _fake_click
+from mne.viz.utils import _fake_click, _fake_keypress
 
 
 def _annotation_helper(raw, browse_backend, events=False):
@@ -334,6 +337,9 @@ def test_plot_raw_selection(raw, browser_backend):
     # test lasso
     # Testing lasso-interactivity of sensor-plot within Qt-backend
     # with QTest doesn't seem to work.
+    want = ['MEG 0111', 'MEG 0112', 'MEG 0113', 'MEG 0131', 'MEG 0132',
+            'MEG 0133']
+    assert want == sorted(fig.mne.ch_names[fig.mne.picks])
     want = ['MEG 0121', 'MEG 0122', 'MEG 0123']
     if ismpl:
         sel_fig._set_custom_selection()  # lasso empty → should do nothing
@@ -727,7 +733,9 @@ def test_plot_raw_psd(raw, raw_orig):
     raw_unchanged = raw.copy()
     # normal mode
     fig = raw.plot_psd(average=False)
-    fig.canvas.resize_event()
+    fig.canvas.callbacks.process(
+        'resize_event',
+        backend_bases.ResizeEvent('resize_event', fig.canvas))
     # specific mode
     picks = pick_types(raw.info, meg='mag', eeg=False)[:4]
     raw.plot_psd(tmax=None, picks=picks, area_mode='range', average=False,
@@ -810,11 +818,12 @@ def test_plot_raw_psd(raw, raw_orig):
     plt.close('all')
 
     # gh-7631
-    data = 1e-3 * np.random.rand(2, 100)
-    info = create_info(['CH1', 'CH2'], 100)
+    n_times = sfreq = n_fft = 100
+    data = 1e-3 * np.random.rand(2, n_times)
+    info = create_info(['CH1', 'CH2'], sfreq)  # ch_types defaults to 'misc'
     raw = RawArray(data, info)
     picks = pick_types(raw.info, misc=True)
-    raw.plot_psd(picks=picks, spatial_colors=False)
+    raw.plot_psd(picks=picks, spatial_colors=False, n_fft=n_fft)
     plt.close('all')
 
 
@@ -848,8 +857,8 @@ def test_plot_sensors(raw):
     assert fig.lasso.selection == []
     _fake_click(fig, ax, (0.65, 1), xform='ax', kind='motion')
     _fake_click(fig, ax, (0.65, 0.7), xform='ax', kind='motion')
-    fig.canvas.key_press_event('control')
-    _fake_click(fig, ax, (0, 0.7), xform='ax', kind='release')
+    _fake_keypress(fig, 'control')
+    _fake_click(fig, ax, (0, 0.7), xform='ax', kind='release', key='control')
     assert fig.lasso.selection == ['MEG 0121']
 
     # check that point appearance changes
@@ -858,11 +867,11 @@ def test_plot_sensors(raw):
     assert (fc[:, -1] == [0.5, 1., 0.5]).all()
     assert (ec[:, -1] == [0.25, 1., 0.25]).all()
 
-    _fake_click(fig, ax, (0.7, 1), xform='ax', kind='motion')
+    _fake_click(fig, ax, (0.7, 1), xform='ax', kind='motion', key='control')
     xy = ax.collections[0].get_offsets()
-    _fake_click(fig, ax, xy[2], xform='data')  # single selection
+    _fake_click(fig, ax, xy[2], xform='data', key='control')  # single sel
     assert fig.lasso.selection == ['MEG 0121', 'MEG 0131']
-    _fake_click(fig, ax, xy[2], xform='data')  # deselect
+    _fake_click(fig, ax, xy[2], xform='data', key='control')  # deselect
     assert fig.lasso.selection == ['MEG 0121']
     plt.close('all')
 
@@ -932,3 +941,35 @@ def test_clock_xticks(raw, dur, n_dec, browser_backend):
     assert tick_texts[0].startswith('19:01:53')
     if len(tick_texts[0].split('.')) > 1:
         assert len(tick_texts[0].split('.')[1]) == n_dec
+
+
+def test_plotting_order_consistency():
+    """Test that our internal variables have some consistency."""
+    pick_data_set = set(_PICK_TYPES_DATA_DICT)
+    pick_data_set.remove('meg')
+    pick_data_set.remove('fnirs')
+    missing = pick_data_set.difference(set(_DATA_CH_TYPES_ORDER_DEFAULT))
+    assert missing == set()
+
+
+def test_plotting_temperature_gsr(browser_backend):
+    """Test that we can plot temperature and GSR."""
+    data = np.random.RandomState(0).randn(2, 1000)
+    data[0] += 37  # deg C
+    # no idea what the scale should be for GSR
+    info = create_info(2, 1000., ['temperature', 'gsr'])
+    raw = RawArray(data, info)
+    fig = raw.plot()
+    tick_texts = fig._get_ticklabels('y')
+    assert len(tick_texts) == 2
+
+
+@pytest.mark.pgtest
+def test_plotting_memory_garbage_collection(raw, pg_backend):
+    """Test that memory can be garbage collected properly."""
+    pytest.importorskip('mne_qt_browser', minversion='0.4')
+    raw.plot().close()
+    import mne_qt_browser
+    from mne_qt_browser._pg_figure import MNEQtBrowser
+    assert len(mne_qt_browser._browser_instances) == 0
+    _assert_no_instances(MNEQtBrowser, 'after closing')
